@@ -38,11 +38,12 @@ or something like that
 
 """
 
-import sys
+import sys, os
 import ConfigParser
 import re
 
 import logging
+import logging.config
 log = logging.getLogger(__name__)
 
 class Error(Exception):
@@ -56,6 +57,7 @@ class MetaConfig(object):
 
     def add_config_file(self, name, path, ConfigClass=DEFAULT_CONFIG_PARSER):
 
+        log.info('Adding config %s from path %s' % (name, path)) 
         conf = ConfigClass()
         conf.read([path])
 
@@ -64,6 +66,7 @@ class MetaConfig(object):
 
     def add_config_fh(self, name, fileobj, ConfigClass=DEFAULT_CONFIG_PARSER):
 
+        log.info('Adding config %s from file object' % name)
         conf = ConfigClass()
         conf.readfp(fileobj)
 
@@ -76,6 +79,8 @@ class MetaConfig(object):
             config_parser.__config_name__ = name
             self._configs[name] = config_parser
 
+        log.info('Config %s added' % name)
+
         return config_parser
 
     def get_config(self, name, ConfigClass=DEFAULT_CONFIG_PARSER, inherit=True):
@@ -106,6 +111,8 @@ class MetaConfig(object):
     def from_config(klass, config_parser):
         mf = klass()
 
+        mf._setup_includes(config_parser)
+        mf._setup_logging(config_parser)
         mf._parse_nested_configs(config_parser)
         mf._parse_external_configs(config_parser)
 
@@ -166,11 +173,41 @@ class MetaConfig(object):
         """
         Parse external config files referenced in metaconfig.conf.
         """
-        pass
-        #!FIXME: need to name each config.
-        #if not config_parser.has_option('metaconfig', 'config-files'):
-        #    return
-        #
-        #config_files = config_parser.get('metaconfig', 'config-files').split()
-        #for cf in config_files:
-        #    self.add_config_file(cf)
+        if not config_parser.has_option('metaconfig', 'config-files'):
+            return
+
+        secname = config_parser.get('metaconfig', 'config-files')
+        for opt in config_parser.options(secname):
+            filename = config_parser.get(secname, opt)
+            log.info('Reading external config %s from file %s' %
+                     (opt, filename))
+            self.add_config_file(opt, filename)
+    
+
+    def _setup_includes(self, config_parser):
+        """
+        Include external metaconfig files.
+
+        """
+        if not config_parser.has_option('metaconfig', 'include'):
+            return
+
+        includes = config_parser.get('metaconfig', 'include').split()
+        for include in includes:
+            log.info("Including metaconfig: %s" % include)
+            config_parser.read(include)
+                     
+
+    def _setup_logging(self, config_parser):
+        """
+        Initialise logging from a nested config.
+
+        """
+        if not config_parser.has_option('metaconfig', 'logging'):
+            return
+
+        logging_file = config_parser.get('metaconfig', 'logging')
+        logging_file = os.path.expanduser(logging_file)
+
+        logging.config.fileConfig(logging_file)
+        log.info('Logging configuration initialised from %s' % logging_file)
